﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System.Runtime.InteropServices;

namespace FontRendering
{
    

    public class GlyphBatch
    {
        GraphicsDevice Device;
        Matrix projection;
        Effect current_effect;
        List<VertexPositionTexture> verts = new List<VertexPositionTexture>();
        

        public GlyphBatch(GraphicsDevice device)
        {
            Device = device;

            projection = Matrix.CreateOrthographicOffCenter(0, device.Viewport.Width, device.Viewport.Height, 0, 0.1f, 1);

        }

        public void Start(Texture2D texture, Effect effect, Color color)
        {
            current_effect = effect;
            RasterizerState stat = new RasterizerState();
            stat.CullMode = CullMode.None;

            Device.BlendState = BlendState.NonPremultiplied;
            Device.DepthStencilState = DepthStencilState.Default;
            SamplerState ss = new SamplerState();
            ss.AddressU = TextureAddressMode.Clamp;
            ss.AddressV = TextureAddressMode.Clamp;
            ss.Filter = TextureFilter.Anisotropic;

            current_effect.Parameters["colour"].SetValue(color.ToVector4());
            current_effect.Parameters["Texture"].SetValue(texture);
            current_effect.Parameters["Projection"].SetValue(projection);
            Device.SamplerStates[0] = ss;
            Device.RasterizerState = stat;
        }
        public void StartSprite(Texture2D texture, Effect effect, Color color)
        {
            current_effect = effect;
            RasterizerState stat = new RasterizerState();
            stat.CullMode = CullMode.None;

            Device.BlendState = BlendState.NonPremultiplied;
            Device.DepthStencilState = DepthStencilState.Default;
            SamplerState ss = new SamplerState();
            ss.AddressU = TextureAddressMode.Wrap;
            ss.AddressV = TextureAddressMode.Wrap;
            ss.Filter = TextureFilter.Anisotropic;

            current_effect.Parameters["colour"].SetValue(color.ToVector4());
            current_effect.Parameters["Texture"].SetValue(texture);
            current_effect.Parameters["Projection"].SetValue(projection);
            Device.SamplerStates[0] = ss;
            Device.RasterizerState = stat;
        }

        public void Draw(Vector4 src, Vector4 dst)
        {
            VertexPositionTexture v1 = new VertexPositionTexture();
            v1.Position = new Vector3(dst.X, dst.Y, -0.5f);
            v1.TextureCoordinate = new Vector2(src.X, src.Y);
            verts.Add(v1);

            v1 = new VertexPositionTexture();
            v1.Position = new Vector3(dst.X + dst.Z, dst.Y, -0.5f);
            v1.TextureCoordinate = new Vector2(src.X + src.Z, src.Y);
            verts.Add(v1);

            v1 = new VertexPositionTexture();
            v1.Position = new Vector3(dst.X, dst.Y + dst.W, -0.5f);
            v1.TextureCoordinate = new Vector2(src.X, src.Y + src.W);
            verts.Add(v1);

            v1 = new VertexPositionTexture();
            v1.Position = new Vector3(dst.X + dst.Z, dst.Y, -0.5f);
            v1.TextureCoordinate = new Vector2(src.X + src.Z, src.Y);
            verts.Add(v1);

            v1 = new VertexPositionTexture();
            v1.Position = new Vector3(dst.X + dst.Z, dst.Y + dst.W, -0.5f);
            v1.TextureCoordinate = new Vector2(src.X + src.Z, src.Y + src.W);
            verts.Add(v1);

            v1 = new VertexPositionTexture();
            v1.Position = new Vector3(dst.X, dst.Y + dst.W, -0.5f);
            v1.TextureCoordinate = new Vector2(src.X, src.Y + src.W);
            verts.Add(v1);

        }

        public void End()
        {
            if (verts.Count == 0)
                return;

            VertexPositionTexture[] lverts = verts.ToArray();
            RasterizerState stat = new RasterizerState();
            stat.CullMode = CullMode.None;
            Device.BlendState = BlendState.NonPremultiplied;
            Device.DepthStencilState = DepthStencilState.None;
            
            foreach (EffectPass pass in current_effect.CurrentTechnique.Passes)
            {
                pass.Apply();
                Device.DrawUserPrimitives<VertexPositionTexture>(PrimitiveType.TriangleList, lverts, 0, verts.Count / 3);
            }
            verts.Clear();
            
        }

    }
}
