﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace FontRendering
{
    /// <summary>
    /// This is the main type for your game.
    /// </summary>
    public class Game1 : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D Buttons;
        Texture2D MouseTexture;
        Vector2 mp;

        enum Modes
        {
            Normal, 
            Outline,
            Inverse,
            DropShadow
        }
        Modes displaymode = Modes.Normal;

        string[] VerdanaMessages = new string[]
        {
            "Verdana scale 2.00",
            "Verdana scale 1.00",
            "Verdana scale 0.75",
            "Verdana scale 0.50"

        };

        string[] VerdanaBoldMessages = new string[]
        {
            "Verdana Bold scale 1.25",
            "Verdana Bold scale 1.00",
            "Verdana Bold scale 0.75",
            "Verdana Bold scale 0.50"

        };

        Rectangle[] ButtonSrc = new Rectangle[]
        {
            new Rectangle(0,0,128,32),
            new Rectangle(0,35,128,32),
            new Rectangle(0,70,128,32),
            new Rectangle(0,105,128,32)
        };

        Rectangle[] ButtonDest = new Rectangle[]
        {
            new Rectangle(284,1000,128,32),
            new Rectangle(692,1000,128,32),
            new Rectangle(1100,1000,128,32),
            new Rectangle(1508,1000,128,32)
        };

        MouseState oldstate;

        #region Font rendering assets
        Texture2D VerdanaTexture;
        Texture2D VerdanaBoldTexture;

        SDFFont[] Verdana = new SDFFont[4];
        SDFFont[] VerdanaBold = new SDFFont[4];
        SDFFont[] VerdanaO = new SDFFont[4];
        SDFFont[] VerdanaBoldO = new SDFFont[4];
        SDFFont[] VerdanaI = new SDFFont[4];
        SDFFont[] VerdanaBoldI = new SDFFont[4];
        SDFFont[] VerdanaD = new SDFFont[4];
        SDFFont[] VerdanaBoldD = new SDFFont[4];

        Effect FontShader;
        Effect OutlineShader;
        Effect InverseShader;
        Effect DropShadow;
        #endregion


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 1920;
            graphics.PreferredBackBufferHeight = 1080;
            graphics.GraphicsProfile = GraphicsProfile.HiDef;
            
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Buttons = Content.Load<Texture2D>(@"buttons");
            MouseTexture = Content.Load<Texture2D>(@"mouse");

            FontShader = Content.Load<Effect>(@"Fonts\SDFFont2");
            OutlineShader = Content.Load<Effect>(@"Fonts\Outline");
            InverseShader = Content.Load<Effect>(@"Fonts\Inverse");
            DropShadow = Content.Load<Effect>(@"Fonts\Drop");


            VerdanaTexture = Content.Load<Texture2D>(@"Fonts\Verdana");
            VerdanaBoldTexture = Content.Load<Texture2D>(@"Fonts\VerdanaBold");

            Verdana[0] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, FontShader, 2.0f);
            Verdana[1] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, FontShader, 1.0f);
            Verdana[2] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, FontShader, 0.75f);
            Verdana[3] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, FontShader, 0.5f);

            VerdanaBold[0] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, FontShader, 1.25f);
            VerdanaBold[1] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, FontShader, 1.0f);
            VerdanaBold[2] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, FontShader, 0.75f);
            VerdanaBold[3] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, FontShader, 0.5f);

            VerdanaO[0] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, OutlineShader, 2.0f);
            VerdanaO[1] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, OutlineShader, 1.0f);
            VerdanaO[2] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, OutlineShader, 0.75f);
            VerdanaO[3] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, OutlineShader, 0.5f);

            VerdanaBoldO[0] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, OutlineShader, 1.25f);
            VerdanaBoldO[1] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, OutlineShader, 1.0f);
            VerdanaBoldO[2] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, OutlineShader, 0.75f);
            VerdanaBoldO[3] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, OutlineShader, 0.5f);

            VerdanaI[0] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, InverseShader, 2.0f);
            VerdanaI[1] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, InverseShader, 1.0f);
            VerdanaI[2] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, InverseShader, 0.75f);
            VerdanaI[3] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, InverseShader, 0.5f);

            VerdanaBoldI[0] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, InverseShader, 1.25f);
            VerdanaBoldI[1] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, InverseShader, 1.0f);
            VerdanaBoldI[2] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, InverseShader, 0.75f);
            VerdanaBoldI[3] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, InverseShader, 0.5f);

            VerdanaD[0] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, DropShadow, 2.0f);
            VerdanaD[1] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, DropShadow, 1.0f);
            VerdanaD[2] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, DropShadow, 0.75f);
            VerdanaD[3] = new SDFFont(GraphicsDevice, @"Content\Fonts\Verdana.txt", VerdanaTexture, DropShadow, 0.5f);

            VerdanaBoldD[0] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, DropShadow, 1.25f);
            VerdanaBoldD[1] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, DropShadow, 1.0f);
            VerdanaBoldD[2] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, DropShadow, 0.75f);
            VerdanaBoldD[3] = new SDFFont(GraphicsDevice, @"Content\Fonts\VerdanaBold.txt", VerdanaBoldTexture, DropShadow, 0.5f);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// game-specific content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                Exit();

            MouseState ms = Mouse.GetState();
            mp = new Vector2(ms.X, ms.Y);
            if ((ms.LeftButton == ButtonState.Pressed) && (oldstate.LeftButton == ButtonState.Released))
            {

                if (ButtonDest[0].Contains(mp))
                    displaymode = Modes.Normal;

                if (ButtonDest[1].Contains(mp))
                    displaymode = Modes.Outline;

                if (ButtonDest[2].Contains(mp))
                    displaymode = Modes.DropShadow;

                if (ButtonDest[3].Contains(mp))
                    displaymode = Modes.Inverse;

            }
            oldstate = ms;
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            Vector2 pos = new Vector2(20, 100);

            SDFFont[] first = Verdana;
            SDFFont[] second = VerdanaBold;

            switch (displaymode)
            {
                case Modes.Normal:
                    {
                        first = Verdana;
                        second = VerdanaBold;
                    }
                    break;

                case Modes.Outline:
                    {
                        first = VerdanaO;
                        second = VerdanaBoldO;
                    }
                    break;

                case Modes.DropShadow:
                    {
                        first = VerdanaD;
                        second = VerdanaBoldD;
                    }
                    break;
                case Modes.Inverse:
                    {
                        first = VerdanaI;
                        second = VerdanaBoldI;
                    }
                    break;

            }
            for (int i = 0; i < 4; i++)
            {
                first[i].StartFontRendering(Color.White);
                first[i].DrawString(VerdanaMessages[i], pos);
                first[i].EndFontRendering();

                pos.Y += 5;
                pos.Y += first[i].MeasureString("V").Y;

            }
            pos.Y += 150;
            for (int i = 0; i < 4; i++)
            {
                second[i].StartFontRendering(Color.White);
                second[i].DrawString(VerdanaBoldMessages[i], pos);
                second[i].EndFontRendering();

                pos.Y += 5;
                pos.Y += second[i].MeasureString("V").Y;

            }

            spriteBatch.Begin();
            for (int i=0; i<4; i++)
            {
                spriteBatch.Draw(Buttons, ButtonDest[i], ButtonSrc[i], Color.White);
            }
            spriteBatch.End();

            spriteBatch.Begin();
            spriteBatch.Draw(MouseTexture, mp, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
